import discord
from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List
from cpl_translation import TranslatePipe
from discord import ui, TextStyle

from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.logging.command_logger import CommandLogger
from bot_data.abc.statistic_repository_abc import StatisticRepositoryABC
from bot_data.model.server import Server
from bot_data.model.statistic import Statistic


class AddStatisticForm(ui.Modal):

    description = ui.TextInput(label='Beschreibung', required=True)
    code = ui.TextInput(label='Code', required=True, style=TextStyle.long)

    def __init__(
            self,
            server: Server,
            stats: StatisticRepositoryABC,
            db: DatabaseContextABC,
            name: str,
            message_service: MessageServiceABC,
            logger: CommandLogger,
            t: TranslatePipe,
            code: str = None,
            description: str = None,
    ):
        ui.Modal.__init__(self, title=name)

        self._server = server
        self._stats = stats
        self._db = db
        self._name = name
        self._message_service = message_service
        self._logger = logger
        self._t = t

        if code is not None:
            self.code.default = code

        if description is not None:
            self.description.default = description

    async def on_submit(self, interaction: discord.Interaction):
        statistic = self._stats.get_statistics_by_server_id(self._server.server_id).where(lambda s: s.name == self._name).single_or_default()

        if interaction.guild is None:
            if statistic is None:
                await self._message_service.send_interaction_msg(interaction, self._t.transform('modules.stats.add.failed'))
            else:
                await self._message_service.send_interaction_msg(interaction, self._t.transform('modules.stats.edit.failed'))
            return

        try:
            if statistic is None:
                self._stats.add_statistic(Statistic(self._name, self.description.value, self.code.value, self._server))
                self._db.save_changes()
                await self._message_service.send_interaction_msg(interaction, self._t.transform('modules.stats.add.success'))
                return

            statistic.description = self.description.value
            statistic.code = self.code.value
            self._stats.update_statistic(statistic)
            self._db.save_changes()
            await self._message_service.send_interaction_msg(interaction, self._t.transform('modules.stats.edit.success'))
        except Exception as e:
            self._logger.error(__name__, f'Save statistic {self._name} failed', e)
            if statistic is None:
                await self._message_service.send_interaction_msg(interaction, self._t.transform('modules.stats.add.failed'))
            else:
                await self._message_service.send_interaction_msg(interaction, self._t.transform('modules.stats.edit.failed'))

        self._logger.trace(__name__, f'Finished stats command form')
